/*
	Advanved Dialogs 1.0.7
	Copyright (C) 2005-2008 Jonas Gehring

	Advanced Dialogs is free software; you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Advanced Dialogs is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


// Header File
// Created 26.01.05; 13:18:51


#ifndef _ADVDIALOGS_H 
#define _ADVDIALOGS_H


/*! \file AdvDialogs.h
 *  \brief This file bundles all functions offered by Advanced Dialogs
 *
 * 	This is the only file you will need when working with Advanced Dialogs, since it has got prototypes
 * 	of all the function offered by this library and data structure definitions for the types that
 * 	are used.
 */


// Neccessary header files from TIGCCLIB
#include <alloc.h>								// Routines for dynamic memory allocation
#include <asmtypes.h>								// Common type definitions known to ASM programmers
#include <compat.h>								// Various calc-dependent and AMS-dependent pseudo-constants
#include <ctype.h>								// Routines for checking and changing character types
#include <graph.h>								// Common (non-windowed) graphic routines
#include <intr.h>								// Routines for creating interrupt handlers
#include <kbd.h>								// Keyboard handling routines
#include <string.h>								// ANSI-compatible string manipulation routines
#include <system.h>								// System routines (for accessing system timers, queues, etc.)


// Start of text
#define TXTSTART 35

#define NEXTTAB -2

//! Just a short macro to make working with grayscales easier
#define GRAYPLANES GrayGetPlane(DARK_PLANE), GrayGetPlane(LIGHT_PLANE)



/*_________Enumerations and Text_________*/


//! Describes valid input modes for InputBoxes
enum _inputModes
{
	//! All alphanumeric characters are allowed
	INPUT_STR,
	//! Only numbers (0-9) are allowed
	INPUT_INT
};

//! Describes vaild text attributes
enum _textAttributes
{
	//! Standard (left-aligned) text
	TXT_STANDARD,
	//! Right-aligned text
	TXT_ALIGNRIGHT,
	//! Centered text
	TXT_CENTERED
};

//! Describes valid buttons for an advanced dialog
enum _buttonTypes
{
	//! No button 
	B_NONE = 0,
	//! Button with text "OK" 
	B_OK,
	//! Button with text "ESC" 
	B_ESC,
	//! Button with text "YES" 
	B_YES,
	//! Button with text "NO" 
	B_NO,
};


/*_______________Typedefs________________*/

//! Convenience type for unsigned char
typedef unsigned char uchar;



/*_______________Structures______________*/

//! Structure for a dialog element
typedef struct 
{
	//! Element type
	uchar element;
	//! Text
	char text[50];
	//! Text color
	uchar color;
	//! Buffer for InputBox
	char *inputbuffer;
	//! Itemlist for DropDown-menus
	const char **itemlist;
	//! Selected item for DropDown
	short *selected;
	//! Maximal chars (or numbers); for InputBox only
	short inputsize;
	//! Custom element flag: Input mode for InputBox; number of items for DropDown; attribute for Text; checked for CheckBox
	uchar flag;
}
ADVEL;

//! Structure for a bitmap
typedef struct
{
	//! X-Position
	short x;
	//! Y-Position
	short y;
	//! BITMAP data for the light and dark plane
	BITMAP *data;
}
ADVBITMAP;

//! Structure for a dialog tab
typedef struct
{
	//! Elements
	ADVEL elements[6];
	//! Bitmap pointer
	ADVBITMAP *bitmaps[6];
	//! Number of Bitmaps in this tab
	uchar numbitmaps;
	//! Numer of elements in this tab
	uchar numelements;
	//! Title of the tab
	char title[40];
}
ADVTAB;

//! Structure for a dialog
typedef struct
{
	//! Max. 6 Tabs can be included in a Dialg
	ADVTAB tabs[6];
	//! Number of tabs in this dialog
	uchar numtabs;
	//! Title of the dialog
	char title[50];
	//! X-Position of the upper left corner
	short left_x;
	//! Y-Position of the upper left corner
	short top_y;
	//! X-Position of the lower right corner
	short right_x;
	//! Y-Position of the lower right corner
	short bottom_y;
	//! Dialog buttons (<= 2)
	uchar buttons[2];
	//! Tells whether the screen should be restored after executing the dialog
	BOOL savescreen;
	//! Interrupt handler for AUTO_INT_1
	INT_HANDLER inthandler;
}
ADVDIALOG;



/*________________Globals________________*/

//! CheckBox indicators
extern BOOL AD_CHECKED[6][6];



/*___________Dialog functions____________*/

/*! \brief Creates and allocates space for a new dialog
 *
 *  This function allocates space for a dialog and sets up a few main arguments. \n
 *  \note You should always check the result of \p AdvDlgNew() to make sure that there is enough memory available. If
 *  the function returns \p NULL you should not proceed with building the dialog.
 *  \param width The width of the dialog
 *  \param height The height of the dialog
 *  \param title The title of the dialog
 *  \param savescreen \p TRUE if the screen should be restored after executing the dialog, \p FALSE otherwise
 *  \returns A pointer to the dialog that was created. If the creation fails \p NULL will be returned.
 */
ADVDIALOG *AdvDlgNew(short width, short height, const char *title, BOOL savescreen);

/*! \brief Adds a Tab to a dialog
 *
 *  \param dialog Pointer to the dialog created with \p AdvDlgNew 
 *  \param tab The number of the new tab
 *  \param title The title of the new tab
 */
void AdvDlgAddTab(ADVDIALOG *dialog, uchar tab, const char *title);

/*! \brief Adds a Text element to a dialog
 *
 *  \sa _textAttributes 
 *  \param dialog Pointer to the dialog created with \p AdvDlgNew() 
 *  \param tab The tab to which the Text should be added
 *  \param line The line in a tab where the Text should be added
 *  \param text The text to be displayed
 *  \param attr Text attribute
 *  \param color Color of the text. This can be either \p COLOR_BLACK or \p COLOR_WHITE 
 */
void AdvDlgAddText(ADVDIALOG *dialog, uchar tab, uchar line, const char *text, uchar attr, uchar color);

/*! \brief Adds an InputBox element to a dialog
 *
 *  \sa _inputModes
 *  \param dialog Pointer to the dialog created with \p AdvDlgNew() 
 *  \param tab The tab to which the InputBox should be added
 *  \param line The line in a tab where the InputBox should be added
 *  \param text Text to be display on the left side of the input field
 *  \param buffer Buffer to store the input from the user
 *  \param maxsize Maximal number of characters that can be entered
 *  \param mode Input mode of the InputBox 
 *  \param color Color of the text. This can be either \p COLOR_BLACK or \p COLOR_WHITE 
 */
void AdvDlgAddInputBox(ADVDIALOG *dialog, uchar tab, uchar line, const char *text, char *buffer, short maxsize, uchar mode, uchar color);

/*! \brief Adds a CheckBox element to a dialog
 *
 *  \param dialog Pointer to the dialog created with \p AdvDlgNew() 
 *  \param tab The tab to which the CheckBox should be added
 *  \param line The line in a tab where the CheckBox should be added
 *  \param text Text to be display on the left side of the CheckBox 
 *  \param checked Initial state of the checkbox (\p TRUE means checked, \p FALSE means unchecked)
 *  \param color Color of the text. This can be either \p COLOR_BLACK or \p COLOR_WHITE 
 */
void AdvDlgAddCheckBox(ADVDIALOG *dialog, uchar tab, uchar line, const char *text, BOOL checked, uchar color);

/*! \brief Adds a DropDown menu to a dialog
 *
 *  \param dialog Pointer to the dialog created with \p AdvDlgNew() 
 *  \param tab The tab to which the DropDown menu should be added
 *  \param line The line in a tab where the DropDown menu should be added
 *  \param text Text to be display on the left side of the DropDown menu
 *  \param itemlist Pointer to an array of strings that contains the text for the items
 *  \param numitems Number of items in itemlist 
 *  \param selected Indicates which item will be selected by default
 *  \param color Color of the text. This can be either \p COLOR_BLACK or \p COLOR_WHITE 
 */
void AdvDlgAddDropDown(ADVDIALOG *dialog, uchar tab, uchar line, const char *text, const char **itemlist, short numitems, short *selected, uchar color);

/*! \brief Adds a button to a dialog
 *
 *  \sa _buttonTypes 
 *  \param dialog Pointer to the dialog created with \p AdvDlgNew() 
 *  \param pos Position of the button (\p 0 for left, \p 1 for right)
 *  \param button Type of the button
 */
void AdvDlgAddButton(ADVDIALOG *dialog, uchar pos, uchar button);

/*! \brief Adds a Bitmap to a dialog
 *
 *  The bitmap will be clipped inside the dialog. The positions are relative to the the dialog positions on the screen.
 *  You can add up to 6 bitmaps to a tab. If this function returns -1, there are already 6 elements in this dialog
 *  \param dialog Pointer to the dialog created with \p AdvDlgNew() 
 *  \param tab The tab to which the Bitmap should be added
 *  \param x X-Position of the Bitmap (relative to the Dialog)
 *  \param y Y-Position of the Bitmap (relative to the Dialog)
 *  \param bitmap Pointer to the Bitmap created with \p AdvDlgBitmapNew()
 *  \returns The number of bitmaps in the selected tab, -1 if there is no more space for bitmaps in the selected tab
 */
char AdvDlgAddBitmap(ADVDIALOG *dialog, uchar tab, short x, short y, ADVBITMAP *bitmap);

/*!  \brief Creates a new Bitmap
 *
 *  \note You should always check the result of \p AdvDlgBitmapNew() to make sure that there is enough memory
 *  available. If the function returns \p NULL you, should not add the bitmap to dialog. 
 *  \note The bitmap will be freed automatically if you call \p AdvDlgFree().
 *  \returns A pointer to the new Bitmap, or \p NULL in case of an error
 */
ADVBITMAP *AdvDlgBitmapNew(short width, short height, const unsigned char *lpdata, const unsigned char *dpdata);

/*! \brief Executes a Dialog
 *
 *  \param dialog Pointer to the dialog created with \p AdvDlgNew()
 *  \param tempint1 \p AUTO_INT_1 (see \ref index "Important notes" for more information)
 *  \returns \p TRUE if the user closed the dialog by pressing [ENTER], \p FALSE if the user pressed [ESC]
 */
BOOL AdvDlgDo(ADVDIALOG *dialog, INT_HANDLER tempint1);

/*! \brief Frees them memory used by a dialog 
 *
 *  \p AdvDlgFree() simply frees the pointer using \p free() and checks the dialog for Bitmaps. If
 *  Bitmaps are found, they are also freed.
 *  \param dialog Pointer to the dialog created with \p AdvDlgNew() 
 */
void AdvDlgFree(ADVDIALOG *dialog);


/*! \brief Shows a MessageBox
 *
 *  Displays a MessageBox and waits for a keypress. The screen is being saved before showing the MessageBox
 *  and restored after the user has pressed a key.
 *  \param title Title of the MessageBox
 *  \param msg Text to be displayed
 *  \param textcolor Color of the text. This can either be \p COLOR_BLACK or \p COLOR_WHITE
 *  \param button Button at the lower right corner
 *  \param tempint1 \p AUTO_INT_1 (see \ref index "Important notes" for more information)
 *  \returns The key pressed by the user
 */
short AdvDlgMessageBox(const char *title, const char *msg, uchar textcolor, uchar button, INT_HANDLER tempint1);



/*_________ProgressBar functions_________*/

/*! \brief Initializes a progress bar
 *
 *  This function will initialize and draw a ProgressBar. Use \p AdvDlgProgressUp() to update the ProgressBar.
 *  \param title Title of the ProgressBar
 *  \param msg Text to be displayed
 *  \param steps Number of steps of the ProgressBar
 */
void AdvDlgProgressBar(const char *title, const char *msg, short steps);

/*! \brief Updates the progress bar
 *
 *  This functions will add another "step" to a ProgressBar that has been created with \p AdvDlgProgressBar().
 *  If you call this functions \p steps times (you should have passed \p steps as an argument to \p AdvDlgProgressBar()),
 *  the ProgressBar will be full. \n
 *  If you call this function and the ProgressBar is already full, it will do nothing.
 */
void AdvDlgProgressUp(void);



/*___________Utility functions___________*/

/*! \brief Saves grayscale screen contents
 * 
 *  A small utility function that saves the grayscale screen contents using Advanced Dialogs's temporary screen
 *  buffers. \n
 *  Use \p RestoreScreen() to restore the screen contents.
 *  \note This function is mainly for internal use 
 */
void SaveScreen(void);

/*! \brief Draws a colored string (black or white)
 *
 *  \note This function is mainly for internal use 
 *  \param x X-Position of the string
 *  \param y Y-Position of the string
 *  \param str Text to be drawn
 *  \param color Text color (\p COLOR_BLACK or \p COLOR_WHITE)
 *  \param mode Text mode (\p TXT_NORMAL or \p TXT_CENTERED)
 */
void GrayDrawColorString(short x, short y, const char *str, uchar color, uchar mode);

/*! \brief Restores grayscale screen contents
 *
 *  A small utility function that restores the grayscale screen contents using Advanced Dialogs's temporary screen
 *  buffers. \n
 *  Use \p SaveScreen() to save the screen contents.
 *  \note This function is mainly for internal use 
 */
void RestoreScreen(void);

/*! \brief Clears the keyboard queue
 *  
 *  \note This function is mainly for internal use 
 */
void ClearKbdQueue(void);

/*! \brief Waits for a keypress and stores it in key
 *
 *  \note This function is mainly for internal use 
 */
void GetKey(void *kbq, short *key);



/*___________Misc Definitions____________*/

//! The Version number as a string
#define ADVDLG_VERSION_STR "1.0.7"


#endif



/*! \mainpage Advanced Dialogs
 *
 *	Advanced Dialogs is a cross-platform (AMS and PedroM) static library for TIGCC for creating grayscale dialogs.
 *	It combines the usefulness of AMS Dialogs with the speed of TIGCC and ExtGraph, offering all standard functions
 *	(Title, Buttons, Input, DropDown, MessgaeBox etc.) and additional useful features not offered by AMS
 *	(Multiple Tabs in a dialog, Bitmaps, CheckBoxes, ProgressBars etc.). Providing these features, Advanced Dialogs
 *	allows the easy creating of a fast and beautiful GUI for your TIGCC project.
 *
 *
 *	For a complete function overview, please see \ref AdvDialogs.h
 *
 *	If you are new to Advanced Dialogs, you should read this document for a short introduction.
 *
 *
 * 	<H2>Table of Contents:</H2>
 * 	<OL>
 * 	<LI>\ref screenshots "Screenshots"</LI>
 * 	<LI>\ref include "Howto include Advanced Dialogs in your TIGCC project"</LI>
 * 	<LI>\ref compat "Compability"</LI>
 * 	<LI>\ref navigation "Navigation"</LI>
 * 	<LI>\ref standard "Howto setup a standard dialog"</LI>
 * 	<LI>\ref results "Howto check the results of a dialog"</LI>
 * 	<LI>\ref bitmaps "Bitmaps"</LI>
 * 	<LI>\ref progmsg "ProgressBar and MessageBox"</LI>
 * 	<LI>\ref important "Important notes"
 * 	<OL>
 * 	<LI>\ref interrupt "Interrupt handlers"</LI>
 * 	<LI>\ref misc "Misc. notes"</LI>
 * 	</OL></LI>
 * 	<LI>\ref credits "Credits"</LI>
 * 	<LI>\ref license "License"</LI>
 * 	<LI>\ref contact "Contact"</LI>
 * 	<LI>\ref history "History"</LI>
 *	</OL>
 *
 * 	\anchor screenshots
 *	<H2>1. Screenshots:</H2>
 *	<TABLE>
 *	<TR><TD>
 *	\image html screen01.gif
 *	</TD><TD>
 *	This screenshot is taken from the example 'Simple Dialog', showing a dialog without tabs.
 *	</TD></TR>
 *	<TR><TD>
 *	\image html screen02.gif
 *	</TD><TD>
 *	This screenshot is taken from the example 'Input', shows a dialog with an InputBox and Text. If you press [ENTER], a MessageBox is shown that displays the name that has been entered.
 *	</TD></TR>
 *	<TR><TD>
 *	\image html screen03.gif
 *	</TD><TD>
 *	This screenshot is taken from the example 'ProgressBar'. It simply demonstrates the ProgressBar-feature.
 *	</TD></TR>
 *	<TR><TD>
 *	\image html screen04.gif
 *	</TD><TD>
 *	This screenshot is taken from example 'Multiple Tabs', showing a dialog with 4 tabs. The tabs include different elements: Text, InputBoxes, CheckBoxes and a DropDown menu.
 *	</TD></TR>
 *	<TR><TD>
 *	\image html screen05.gif
 *	</TD><TD>
 *	This screenshot is taken from example "Bitmap", showing a simple dialog with a bitmap and Text with the attribute TXT_ALIGNLEFT (the first two lines).
 *	</TD></TR>
 *	</TABLE>
 *
 *
 *	\anchor include
 *	<H2>2. Howto include Advanced Dialogs in your TIGCC project</H2>
 *	If you want to use Advanced Dialogs in your TIGCC project, you will need to add four files in order to get it working: 
 *	<TABLE>
 *	<TR><TD>
 *	AdvDialogs.h
 *	</TD><TD>
 *	Advanced Dialogs header file
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	AdvDialogs.a
 *	</TD><TD>
 *	Advanced Dialogs archive file (the library)
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	extgraph.h
 *	</TD><TD>
 *	ExtGraph header file
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	extraph.a
 *	</TD><TD>
 *	ExtGraph archive file
 *	</TD></TR>
 *	</TABLE>
 *
 *	The AdvDialogs*-files can be found in the lib-directory of this package. The ExtGraph library can be downloaded at the TI-Chess Team homepage (http://tict.ticalc.org),
 *	allthough it is recommend to get a svn copy of the ExtGraph svn repository at http://opensvn.csie.org/ExtGraph.
 *	Please note that the ExtGraph library and its functions were written by members of the TI-Chess Team and without it, Advanced Dialogs would be useless. 
 *	If you are already using ExtGraph in your project, you do not nedd to include them once more. Just make sure that your version is not older than 2.00beta4. 
 *
 *	To add this files to your project, put them in your source folder, and select "Project->Add Files..." from the TIGCC IDE. 
 *	Last but not least, you need to add something like
 *
 * 	\code
 *	#include "AdvDialogs.h"   // Powered by Advanced Dialogs 1.0.7
 *	\endcode
 *
 *	to the files that will call functions from the Advanced Dialogs library.
 *
 *
 *	\anchor compat 
 *	<H2>3. Compability</H2>
 *	For checking if Advanced Dialogs is compatible to your project and/or calculator, use the following table.
 *
 *	<TABLE>
 *	<TR><TD>
 *	<B>HW 1</B>
 *	</TD><TD>
 *	AMS >= 1.01
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	<B>HW 2</B>
 *	</TD><TD>
 *	AMS >= 1.01
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	<B>HW 3</B>
 *	</TD><TD>
 *	AMS >= 1.01
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	<B>PedroM</B>
 *	</TD><TD>
 *	version >= 0.81
 *	</TD></TR>
 *	</TABLE>
 *
 *	If you find any compability issues that are not listed here, please \ref contact "contact" me!
 *
 *
 *	\anchor navigation
 *	<H2>4. Navigation</H2>
 *	The following table shows the keys and their meanings:
 *
 *	<TABLE>
 *	<TR><TD>
 *	<B>[ENTER]</B>
 *	</TD><TD>
 *	Closes the dialog and returns \p TRUE
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	<B>[ESC]</B>
 *	</TD><TD>
 *	Closes the dialog and returns \p FALSE
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	<B>[APPS]</B>
 *	</TD><TD>
 *	Switches between the tabs from left to right
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	<B>[LEFT]/[RIGHT]</B>
 *	</TD><TD>
 *	(Un)check CheckBoxes; Select an item in a DropDown element
 *	</TD></TR><TR><TD>
 *	<TR><TD>
 *	<B>[UP]/[DOWN]</B>
 *	</TD><TD>
 *	Switches between interactive elements (CheckBox, InputBox and DropDown)
 *	</TD></TR>
 *	</TABLE>
 *
 * 	\anchor standard
 * 	<H2>5. Howto setup a standard dialog</H2>
 * 	Before you use the Advanced Dialogs library, you should understand the structure of the dialogs you can create with it.
 * 	Look at the following picture to get an idea of it:
 * 	\image html structure.png "Dialog structure"
 * 	As you can see, a dialog can contain multiple tabs, while the maximal number is 6. Each tab can contain multiple elements
 * 	(e.g Text or CheckBoxes), while you can store up to 6 elements per tab. This makes a total of 36 elements per dialog. <BR> 
 * 	If you understand this structure, it is easy to set up complex dialogs. All you have to do is keep track of the correct order:
 *	<OL>
 *	<LI>Create a new dialog using \p AdvDlgNew()</LI> 
 * 	<LI>Add a tab to the dialog using \p AdvDlgAddTab()</LI> 
 * 	<LI>Add some elements to the tab</LI>
 * 	</OL>
 * 	To make this more clear, look at the following program that will set up a dialog with two tabs. Each tab contains a line of text.
 *
 * 	\code
 * #include <tigcclib.h> 
 * #include "AdvDlgs.h"   // Powered by Advanced Dialogs 1.0.7
 *
 *
 * void _main (void) 
 * { 
 *  	// Turn on grayscale graphics 
 * 	if (!GrayOn()) { 
 * 		return; 
 * 	}
 *
 * 	// Use the type ADVDIALOG to create a dialog
 * 	ADVDIALOG *dialog = AdvDlgNew(120, 40, "Dialog Test", TRUE); 
 * 
 * 	// Safety first!!
 * 	if (dialog == NULL) { 
 * 		GrayOff(); 
 * 		AdvDlgFree(dialog);
 * 		return; 
 * 	} 
 * 
 * 	// Add the first tab (it is numbered with '0') 
 * 	AdvDlgAddTab(dialog, 0, "Tab No.1"); 
 * 		// Add a line of text to tab '0' in line '1' 
 * 		AdvDlgAddText(dialog, 0, 1, "This is the first tab", TXT_STANDARD, COLOR_BLACK); 
 * 	// Add the second tab (numbered as '1')
 * 	AdvDlgAddTab(dialog, 1, "Tab No.2"); 
 * 		// Add a line of text to tab '1' in line '2' 
 *  		AdvDlgAddText(dialog, 1, 2, "Centered text...", TXT_CENTERED, COLOR_WHITE); 
 * 
 * 	// Buttons can be added any time to a dialog 
 * 	AdvDlgAddButton(dialog, 0, B_OK); 
 * 	AdvDlgAddButton(dialog, 1, B_ESC); 
 *
 * 	// Now, execute the dialog 
 * 	AdvDlgDo(dialog, DUMMY_HANDLER); 
 * 
 * 	// Don't forget to free the dialog after execution! 
 * 	AdvDlgFree(dialog); 
 * 
 * 	// Turn off grayscale graphics
 * 	GrayOff(); 
 * }
 *	\endcode
 *	Easy, isn't it?
 *
 *
 *	\anchor results
 *	<H2>6. Howto check the results of a dialog</H2>
 * 	If your dialog uses interactive elements (InputBoxes, CheckBoxes or DropDowns), you might want to know the values the user selected or entered.
 *
 * 	In InputBoxes, the pointer to the inputbuffer contains the string the user entered.<BR> 
 * 	When using DropDowns, the pointer to the selected argument will contain the selected argument after execution of the dialog.<BR> 
 * 	CheckBoxes are different: you can check their results via a global array named \p AD_CHECKED. So, you could check the content of CheckBox '2' in Tab '0' with:
 * 	\code
 * 	if (AD_CHECKED[0][2])
 *	{
 *		// CheckBox is checked
 *	}
 *	else
 *	{
 *		// CheckBox is not checked
 *	}
 * 	\endcode
 *
 *
 * 	\anchor bitmaps
 * 	<H2>7. Bitmaps</H2>
 *	Starting from version 1.0, you are able to add images to dialogs (tabs), which are called bitmaps. They have their own type
 *	named \p ADVBITMAP. The format of bitmaps is based on the TIOS \p BITMAP format: A variable of type \p ADVBITMAP contains the x-
 *	and the y-postion of the bitmap as well as two variables of type \p BITMAP - one for the light and one for the dark plane.
 *	The bitmaps are drawn using the TIOS- and PedroM-built-in routine \p BitmapPut. 
 *
 *	The x- and y-position of a bitmap are relative to the dialog to which it has been added (that's why they are specified when
 *	you add the bitmap, not when you create it). The bitmap is clipped inside the dialog to which it has been added. 
 *
 *	In this version, it is only possible to add 1 bitmap to a dialog (tab), although future version might enable more bitmaps per dialog (tab).
 *	\sa _advbitmap
 *
 *
 *	\anchor progmsg
 *	<H2>8. ProgressBar and MessageBox</H2>
 *	Beside the standard dialog functions, Advanced Dialogs also offers two other functions that are totally independent from the
 *	dialogs. These functions are \p AdvDlgProgressBar(), which will draw a ProgressBar to add some animation to a process, and
 *	\p AdvDlgMessageBox(), which can be used to display one-line messages with a single call. 
 *
 *	I will explain these functions briefly, but please take a look at the examples to see some code that will make them work.
 *	
 *	<H3>8.1 ProgressBar</H3>
 *	The neccessary function calls to setup a ProgressBar are
 *	<OL>
 *	<LI>Setup the ProgressBar with \p AdvDlgProgressBar()</LI>
 *	<LI>Cal \p AdvDlgProgressUp() until the ProgressBar is full</LI>
 *	</OL>
 *	Please note that the ProgressBar does not save the LCD contents, so you will have to do	this by yourself.
 *
 *	<H3>8.2 MessageBox</H3>
 *	To show a small MessageBox, you simply have to call \p AdvDlgMessageBox() and you will get a small dialog with a title, a button
 *	and one line of text. If the user presses a key, the MessageBox will close and the function returns.
 *	MessageBox saves the LCD contents and restores them before it returns.
 *
 *	\sa AdvDlgProgressBar
 *	\sa AdvDlgMessageBox
 *
 *
 *	\anchor important 
 *	<H2>9. Important notes</H2>
 *	\anchor interrupt
 *	<H3>9.1 Interrupt handlers</H3>
 *	In most grayscale programs, \p AUTO_INT_1 and \p AUTO_INT_5 are disabled to avoid the display of the \p 2ND, \p ALPHA,
 *	or \p DIAMOND -indicators in the status line. Programs that are using low-level keyboard reading should disable these
 *	interrupts as well. 
 *
 *	However, Advanced Dialogs doesn't disable interrupts to make functions like InputBox possible and to keep
 *	compability to PerdroM, which doesn't support \p _rowread. 
 *
 *	Therefore, the functions \p AdvDlgDo() and \p AdvDlgMessageBox() take an interrupt handler as an argument. If your program
 *	disables interrupts, you should use the saved \p AUTO_INT_1 for this argument. If you do so, Advanced Dialogs will use
 *	\p GraySetInt1Handler() to reinstall a temporary \p AUTO_INT_1 and install \p DUMMY_HANDLER before it exits. 
 *
 *	If you do not disable interrupts in your program, you should pass \p DUMMY_HANDLER as an argument.
 *
 *	\anchor misc 
 *	<H3>9.2 Misc. notes</H3>
 *	Another thing that is important when working with Advanced Dialogs is to keep track of the functions you call, because
 *	this library \b won't do error checking of the arguments or of the functions. 
 *	To make sure your program runs as well as it should, you must not exceed the limits of a dialog, i.e do not add more than
 *	6 tabs per dialog, more than 6 elements per tab. \n 
 *	If you don't want to use tabs in your dialog or if you just have one tab, all you have to do is not to call \p AdvDlgAddTab()
 *	and add all elements to tab '0'. \n \n
 *	Advanced Dialogs is released under the GNU Lesser General Public License, so you will find the complete source in the folder "source".
 *	If you have some experience with C, it should be easy to modify the dialog routines to fit your needs. You are completely
 *	free to add your own extensions, or to program a different style etc. Please feel free to send me your modifications - perhaps
 *	I will include them in this package.
 *
 *
 *	\anchor credits
 *	<H2>10. Credits</H2>
 *	\b Advanced \b Dialogs \n 
 *	version 1.0.7 - 05.11.2008 \n \n
 *	written by \n
 *	Jonas Gehring (aka saubue), \n
 *	member of boolsoft \n \n
 *	compiled with TIGCC v0.96 Beta 8 \n 
 *	powered by ExtGraph\n
 *	tested and debugged with TiEmu and VirtualTI \n \n
 *	Beta-Testers:\n 
 *	&nbsp; &nbsp; Epsilon \n
 *	&nbsp; &nbsp; MathStuf \n \n
 *	Thanks to: \n
 *	&nbsp; &nbsp; The TIGCC Team for their wonderful compiler and IDE \n 
 *	&nbsp; &nbsp; The TI-Chess Team for the great ExtGraph library \n
 *	&nbsp; &nbsp; The Linux Programming Group for TiEmu \n
 *	&nbsp; &nbsp; mhubi \n
 *	&nbsp; &nbsp; Epsilon and MathStuf for their suggestions (Epsilon had the great idea to use tabs) \n
 *	&nbsp; &nbsp; Lionel Debroux for his patches and tips \n
 *	&nbsp; &nbsp; Lachprog and Boscop for their useful bug reports \n
 *	&nbsp; &nbsp; All people who support the community and who are trying to keep everything together \n
 *
 *	
 *	\anchor license
 *	<H2>11. License</H2>
 *	Advanved Dialogs 1.0.7 \n
 *	Copyright (C) 2005 - 2008 Jonas Gehring \n
 *	\n
 *	Advanced Dialogs is free software; you can redistribute it and/or modify \n
 *	it under the terms of the GNU Lesser General Public License as published by \n
 *	the Free Software Foundation; either version 2 of the License, or \n
 *	(at your option) any later version. \n
 *	\n 
 *	Advanced Dialogs is distributed in the hope that it will be useful, \n
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of \n
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n
 *	GNU Lesser General Public License for more details. \n
 *	\n
 *	You should have received a copy of the GNU Lesser General Public License \n
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>. \n
 *
 *
 *	\anchor contact
 *	<H2>12. Contact</H2>
 *	If you have any comments or suggestions, if you want to support boolsoft, if you have found a bug or if you just want to say "hello!", please write to: \n \n 
 *	&nbsp; &nbsp; saubue@boolosft.org or \n 
 *	&nbsp; &nbsp; jonas.gehring@boolsoft.org \n \n
 *	For future updates and other programs, please visit our homepage and sign our guestbook: \n \n 
 *	&nbsp; &nbsp; http://www.boolsoft.org \n \n 
 *	For further discussion of Advanced Dialogs and our other projects and programs as well as for minor updates, visit our message board: \n \n
 *	&nbsp; &nbsp; http://board.boolsoft.org \n \n
 *	If you want to meet the biggest german TI community, please visit MobiFiles: \n \n 
 *	&nbsp; &nbsp; http://www.mobifiles.de and its message board at \n  
 *	&nbsp; &nbsp; http://forum.mobifiles.de \n
 *
 *
 *	\anchor history
 *	<H2>13. History</H2>
 *	<TABLE>
 *	<TR><TD style="vertical-align:top"><B>Date</B></TD><TD style="vertical-align:top"><B>Version #</B></TD><TD style="vertical-align:top"><B>Features / Changes</B></TD>
 *
 *	<TR><TD style="vertical-align:top">XX.08.2008</TD><TD style="vertical-align:top">\b 1.0.7</TD><TD>Fixed bug in message box button drawing reported by Boscop \n Added missing dot in version naming scheme</TD></TR>
 *	<TR><TD style="vertical-align:top">02.02.2008</TD><TD style="vertical-align:top">\b 1.06</TD><TD>Fixed a bug reported by Lachprog and reverted DropDown selection colors</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">12.09.2007</TD><TD style="vertical-align:top">\b 1.05</TD><TD>Added patches from Lionel Debroux (thanks!) \n Dialogs can now hold up to 6 bitmaps per tab \n Fixed MessageBox button text color \n ExtGraph is not distributed with Advanced Dialogs any more \n Examples are documented now \n Some small size optimizations</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">04.08.2007</TD><TD style="vertical-align:top">\b 1.04</TD><TD>Advanced Dialogs now uses the GNU Lesser General Public License (LGPL)</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">30.07.2007</TD><TD style="vertical-align:top">\b 1.03</TD><TD>Fixed multiple tabs bug reported by Lachprog</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">10.07.2007</TD><TD style="vertical-align:top">\b 1.02</TD><TD>Fixed font bug reported by Lachprog \n Fixed bug in some examples (not freeing memory if \p GrayOn() fails) \n Updated documentation, now using doxygen</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">22.02.2006</TD><TD style="vertical-align:top">\b 1.01</TD><TD>Fixed silly logical bug - right-aligned text has the attribute \p TXT_ALIGNRIGHT and not \p TXT_ALIGNLEFT, of course...</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">17.01.2006</TD><TD style="vertical-align:top">\b 1.0</TD><TD>Added Bitmap-functions \n Modified x- and y-Position of Elements in dialogs with no tabs \n Added \p TXT_ALIGNLEFT as a text attribute. \n Renamed examples and added another one to demonstrate the new Bitmap-functions</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">19.12.2005</TD><TD style="vertical-align:top">\b 1.0beta</TD><TD></TD>Massive Changes in this near-final version: \n &nbsp;- The construction of the dialogs has changed once again to make multiple tabs possible. They are called like TI-BASIC-Dialogs now. \n &nbsp;- You are able to use tabs now, so up to 36 lines of text, input etc. can be stored into 1 dialog. You can switch tabs by pressing \p [APPS] \n &nbsp;- Inputs can now scroll, so the string is not limited to the size of the InputBox \n &nbsp;- Added DropDown menu \n &nbsp;- Dialogs now have their own type (\p ADVDIALOG). They are initialized as pointers, \p AdvDlgNew() allocates the space for them. \n &nbsp;- Added example #4 to demonstrate the new tab features \n &nbsp;- The whole documentation has been rewritten</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">07.09.2005</TD><TD style="vertical-align:top">\b 0.91</TD><TD>Fixed bug in \p AdvDlgProgressUp() when using a 240x128 screen \n '\' is now also allowed in InputBoxes with \p INPUT_STR</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">29.08.2005</TD><TD style="vertical-align:top">\b 0.9</TD><TD>Finally a nearly complete version (that's the reason for \b 0.9) and a lot of new stuff: \n &nbsp;- Dialogs have to be called totally different. This makes working with them easier and more dynamically. \n &nbsp;- Added CheckBox-feature \n &nbsp;- Improved ProgressBar-feature \n &nbsp;- You can scroll through the interactive elements with \p [UP] and \p [DOWN] \n &nbsp;- added optimizations by MathStuf that made screen centering wasting less bytes</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">26.08.2005</TD><TD style="vertical-align:top">\b 0.5</TD><TD>The standard dialog is variable-sized and centered on the screen</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">22.08.2005</TD><TD style="vertical-align:top">\b 0.46</TD><TD>Bugfixes and optimizations \n Thanks to MathStuf!</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">09.08.2005</TD><TD style="vertical-align:top">\b 0.45</TD><TD>All dialogs and the ProgressBar are now centered on a TI92+/V200, too</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">08.08.2005</TD><TD style="vertical-align:top">\b 0.4</TD><TD>Added MessageBox-function</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">06.08.2005</TD><TD style="vertical-align:top">\b 0.3</TD><TD>Added ProgressBar-functions \n Added example #3</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">12.06.2005</TD><TD style="vertical-align:top">\b 0.22</TD><TD>Fixed bug in example #2 \n Documentation updated</TD></TR>
 *
 *	<TR><TD style="vertical-align:top">20.03.2005</TD><TD style="vertical-align:top">\b 0.21</TD><TD>First release</TD></TR>
 *
 *	</TABLE>
 */


/*! \example adsmpl.c 
 *
 * 	This example shows how to setup a simple dialog
 */


/*! \example adinput.c 
 *
 *	This example shows how Advanced Dialogs can be used to get input from the user. 
 *
 *	A dialog is created as usual and an InputBox is added to it. After the user accepted the dialog, the input string is displayed using a MessageBox.
 */


/*! \example adpbar.c 
 * 
 * 	This example demonstrates the usage of the ProgressBar-feature.
 *
 * 	A ProgressBar is being setup and a dummy process is being shown
 */


/*! \example admtabs.c
 *
 * 	This example shows how to use multiple tabs within a dialog.
 */


/*! \example adbmp.c
 * 
 * 	This example shows a little dialog including a Bitmap
 *
 * 	The Bitmap data is defined in adlogo.h, which is kind of unintersesting now. Here is the example code:
 */

